{
    "$graph": [
        {
            "class": "CommandLineTool",
            "doc": "This tool will run OxoG. The OxoG tool was written by Dimitri Livitz. This CWL wrapper was written by Solomon Shorser.\n",
            "requirements": [
                {
                    "class": "DockerRequirement",
                    "dockerPull": "quay.io/pancancer/pcawg-oxog-filter"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "id": "#oxog.cwl/inputFileDirectory",
                    "type": "Directory",
                    "inputBinding": {
                        "position": 0,
                        "prefix": "--inputDir"
                    }
                },
                {
                    "id": "#oxog.cwl/tumourID",
                    "type": "string",
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--pairID"
                    }
                },
                {
                    "id": "#oxog.cwl/tumourBamFilename",
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--bamName"
                    },
                    "secondaryFiles": ".bai"
                },
                {
                    "id": "#oxog.cwl/oxoQScore",
                    "type": "float",
                    "inputBinding": {
                        "position": 4,
                        "prefix": "--oxoqScore"
                    }
                },
                {
                    "id": "#oxog.cwl/vcfNames",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--vcfs"
                    }
                },
                {
                    "id": "#oxog.cwl/refDataDir",
                    "type": "Directory",
                    "inputBinding": {
                        "position": 7,
                        "prefix": "--refDataDir"
                    }
                }
            ],
            "baseCommand": [
                "python3",
                "/cga/fh/pcawg_pipeline/run_oxog_tool.py"
            ],
            "id": "#oxog.cwl",
            "http://purl.org/dc/terms/creator": {
                "foaf:name": "Solomon Shorser",
                "foaf:mbox": "solomon.shorser@oicr.on.ca"
            },
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "failing_intermediates.tar"
                    },
                    "id": "#oxog.cwl/debuggingOutput"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "output_files/*oxoG.maf"
                    },
                    "id": "#oxog.cwl/oxogMAF"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "output_files/*oxoG.supplementary.tar.gz"
                    },
                    "id": "#oxog.cwl/oxogSupplementaryFile"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "output_files/*oxoG.vcf.gz.tbi"
                    },
                    "id": "#oxog.cwl/oxogTBI"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputBinding": {
                        "glob": "output_files/*oxoG.vcf.gz"
                    },
                    "id": "#oxog.cwl/oxogVCF"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "output_files/*summary.usage.txt"
                    },
                    "id": "#oxog.cwl/usageInfo"
                }
            ]
        },
        {
            "doc": "This is a subworkflow - this is not meant to be run as a stand-alone workflow!\n",
            "requirements": [
                {
                    "class": "SchemaDefRequirement",
                    "types": [
                        {
                            "name": "#TumourType.yaml/TumourType",
                            "type": "record",
                            "fields": [
                                {
                                    "name": "#TumourType.yaml/TumourType/tumourId",
                                    "type": "string",
                                    "inputBinding": {
                                        "position": 0
                                    }
                                },
                                {
                                    "name": "#TumourType.yaml/TumourType/bamFileName",
                                    "type": "string",
                                    "inputBinding": {
                                        "position": 1
                                    }
                                },
                                {
                                    "name": "#TumourType.yaml/TumourType/associatedVcfs",
                                    "type": {
                                        "type": "array",
                                        "items": "string"
                                    },
                                    "inputBinding": {
                                        "position": 2
                                    }
                                },
                                {
                                    "name": "#TumourType.yaml/TumourType/oxoQScore",
                                    "type": "float",
                                    "inputBinding": {
                                        "position": 3
                                    }
                                }
                            ]
                        }
                    ]
                },
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "MultipleInputFeatureRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement",
                    "expressionLib": [
                        "/**\n * Creates an array of files that will be used as input for OxoG.\n * @param in_data This is a TumourType object (see TumourType.yaml), representing a single tumour.\n * @param vcfsForOxoG This is an array of ALL available VCFs, from ALL tumours, as CWL File objects.\n * @return vcfsToUse Is an array of CWL File objects.\n */\nfunction createArrayOfFilesForOxoG(in_data, vcfsForOxoG) {\n\t//TODO: Move this function to separate JS file.\n\tvar vcfsToUse = []\n\t// Need to search through vcfsForOxoG (cleaned VCFs that have been zipped and index) and preprocess_vcfs/extractedSNVs to find VCFs\n\t// that match the names of those in in_data.inputs.associatedVCFs\n\t// associatedVcfs is the VCFs for the tumour represented by `in_data`.\n\t// Note: The data in associatedVcfs is just file names, the data objects in vcfsForOxoG are actual CWL File objects.\n\tvar associatedVcfs = in_data.associatedVcfs\n\t// Loop through the VCFs for this given tumour...\n\tfor (var i in associatedVcfs)\n\t{\n\t\t// We want SNVs, for OxoG\n\t\tif (associatedVcfs[i].indexOf(\".snv\") !== -1)\n\t\t{\n\t\t\t// Loop through ALL available VCFs. If there is a CWL File in vcfsForOxoG whose basename matches the associatedVcf,\n\t\t\t// add the CWL File to the output array (because we need to output CWL File objects, not just file names).\n\t\t\tfor (var j in vcfsForOxoG)\n\t\t\t{\n\t\t\t\tif (vcfsForOxoG[j].basename.indexOf( associatedVcfs[i].replace(\".vcf.gz\",\"\") ) !== -1 && /.*\\.gz$/.test(vcfsForOxoG[j].basename))\n\t\t\t\t{\n\t\t\t\t\tvcfsToUse.push(vcfsForOxoG[j])\n\t\t\t\t}\n\t\t\t}\n\t\t}\n\t\tif (associatedVcfs[i].indexOf(\".indel\") !== -1)\n\t\t{\n\t\t\tfor ( var j in vcfsForOxoG )\n\t\t\t{\n\t\t\t\t// If the associatedVcf is an INDEL, check if there were any SNVs extracted from it, and if so, add to the output array.\n\t\t\t\tif(vcfsForOxoG[j].basename.replace(\".pass-filtered.cleaned.vcf.normalized.extracted-SNVs.vcf.gz\",\"\").indexOf( associatedVcfs[i].replace(\".vcf.gz\",\"\") ) !== -1 && /.*\\.gz$/.test(vcfsForOxoG[j].basename))\n\t\t \t\t{\n\t\t\t\t\tvcfsToUse.push(vcfsForOxoG[j])\n\t\t\t\t}\n\t\t\t}\n\t\t}\n\t}\n\treturn vcfsToUse\n}\n\n/**\n * Flattens nested arrays into a single array.\n * @param array_of_arrays - an array that might contain subarrays as its elements.\n * @return All elements and subelements and sub-subelements (to any level of nesting) flattened into a single array with no nested elements.\n */\nfunction flatten_nested_arrays(array_of_arrays)\n{\n\tvar flattened_array = []\n\tfor (var i in array_of_arrays)\n\t{\n\t\tvar item = array_of_arrays[i]\n\t\tif (item instanceof Array)\n\t\t{\n            // console.log(\"found subarray\")\n\t\t\t// recursively flatten subarrays.\n\t\t\tvar flattened_sub_array = flatten_nested_arrays(item)\n\t\t\tfor (var k in flattened_sub_array)\n\t\t\t{\n                flattened_array.push(flattened_sub_array[k])\n\t\t\t}\n\t\t}\n\t\telse\n\t\t{\n\t\t\tflattened_array.push(item)\n\t\t}\n\t}\n\treturn flattened_array\n}\n"
                    ]
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                }
            ],
            "class": "Workflow",
            "outputs": [
                {
                    "outputSource": "#main/sub_run_oxog/oxogTBI",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/oxogTBI"
                },
                {
                    "outputSource": "#main/sub_run_oxog/oxogVCF",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/oxogVCF"
                }
            ],
            "inputs": [
                {
                    "type": "#TumourType.yaml/TumourType",
                    "id": "#main/in_data"
                },
                {
                    "type": "Directory",
                    "id": "#main/inputFileDirectory"
                },
                {
                    "type": "Directory",
                    "id": "#main/refDataDir"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/vcfsForOxoG"
                }
            ],
            "steps": [
                {
                    "run": "#oxog.cwl",
                    "in": [
                        {
                            "source": "#main/inputFileDirectory",
                            "id": "#main/sub_run_oxog/inputFileDirectory"
                        },
                        {
                            "source": [
                                "#main/in_data"
                            ],
                            "valueFrom": "${\n    return self.oxoQScore\n}\n",
                            "id": "#main/sub_run_oxog/oxoQScore"
                        },
                        {
                            "source": "#main/refDataDir",
                            "id": "#main/sub_run_oxog/refDataDir"
                        },
                        {
                            "source": [
                                "#main/inputFileDirectory",
                                "#main/in_data"
                            ],
                            "valueFrom": "${\n    return { \"class\":\"File\", \"location\": self[0].location + \"/\" + self[1].bamFileName }\n}\n",
                            "id": "#main/sub_run_oxog/tumourBamFilename"
                        },
                        {
                            "source": [
                                "#main/in_data"
                            ],
                            "valueFrom": "${\n    return self.tumourId\n}\n",
                            "id": "#main/sub_run_oxog/tumourID"
                        },
                        {
                            "source": [
                                "#main/in_data",
                                "#main/vcfsForOxoG"
                            ],
                            "valueFrom": "${\n    return createArrayOfFilesForOxoG(self[0], self[1])\n}\n",
                            "id": "#main/sub_run_oxog/vcfNames"
                        }
                    ],
                    "out": [
                        "#main/sub_run_oxog/oxogVCF",
                        "#main/sub_run_oxog/oxogTBI"
                    ],
                    "id": "#main/sub_run_oxog"
                }
            ],
            "id": "#main"
        }
    ],
    "cwlVersion": "v1.0"
}
